// H E R M E S  M A I L
// Copyright 1997-98 OaS 2004
// Further info at http://www.lysator.liu.se/~fcl/hermes/

// $Header: /Hermes GUI/THeader.cpp 20    97-12-10 14:45 Fcl $
// $Log: /Hermes GUI/THeader.cpp $
//
//20    97-12-10 14:45 Fcl
//Added an functioning reply button
//
//19    97-12-07 10:36 Fcl
//Fixed that caused the date to format incorrectly - this caused problems
//when 
//send letters (crash)
//
//18    97-11-30 18:45 Fcl
//Added decoding of the header fields via an DecodeMIMEHeader function
//
//17    97-11-30 11:10 Fcl
//
//16    97-11-28 23:38 Fcl
//Added cut&paste support in header window
//Added copy constructors when needed,
//
//15    97-11-28 14:37 Fcl
//
//14    97-11-26 17:19 Fcl
//Added LetterView class
//
//13    97-11-26 13:04 Peblin
//fixed precompiled headers
//
//12    11/23/97 8:00p Andreas
//Added the date-field in THeader.
//
//11    97-11-23 13:27 Peblin
//Added CSocket support
//
//10    97-11-15 13:12 Bjorn
//Added GetFLags() for TIO
//
//9     97-11-14 19:31 Fcl
//Check in to be able to move the repository to mandel.
//
//8     97-11-07 16:50 Peblin
//Fixed const declarations
//
//7     97-11-07 11:26 Peblin
//
//6     97-11-07 10:31 Peblin
//All functionality implemented. Compiles
//
//5     97-11-06 21:59 Fcl
//Added Header and Log statements at the top of each file

// THeader.cpp: implementation of the THeader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "THeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


THeader::~THeader()
{

}

CTime ConvertDate(CString datestr)
{
	// Not in design
	// Return a time in CTime format (for use when comparing dates)
	char weekday[80], monthstr[80];
	const char months[12][5] = {"Jan",
											"Feb",
											"Mar",
											"Apr",
											"May",
											"Jun",
											"Jul",
											"Aug",
											"Sep",
											"Oct",
											"Nov",
											"Dec"};
 	
	int day, year, hour, minute, second;
	sscanf(datestr, "%s %d %s %d %d:%d:%d", 
							weekday,
							&day,
							monthstr,
							&year,
							&hour,
							&minute,
							&second);

	// Easy way to convert monthstring to monthnumber
	int month = 1;
	while((strcmp(monthstr, months[month-1])) && (month <= 12))
		month++;

	return CTime(year, month, day, hour, minute, 0);
}

THeader::THeader(TAddressList & inTo, TAddress & inFrom, TAddressList & inCc,
				 CString & inSubject, CString& inDate, int inFlags)
{
	to = &inTo;
	from = inFrom;
	subject = inSubject;
	cc = &inCc;
	date = ConvertDate(inDate);
	datestr = inDate;
	flags = inFlags;
}


// Not in design
THeader::THeader(TAddressList & inTo, TAddress & inFrom, TAddressList & inCc,
				 CString & inSubject, CTime& inDate, int inFlags)
{
	to = &inTo;
	from = inFrom;
	subject = inSubject;
	cc = &inCc;
	datestr = inDate.Format("%a, %d %b %Y %H:%M:%S");
	date = inDate;
	flags = inFlags;
}

THeader::THeader(THeader& old)
{
	to = new TAddressList(*old.to);
	from = old.from;
	subject = old.subject;
	cc = new TAddressList(*old.cc);
	date = old.date;
	datestr = old.datestr;
	flags = old.flags;
}

// Not in design
CString DecodeMIMEHeader(CString str)
{
	CString res, word;
	int pos;
	while(str.GetLength() > 0)
	{
		pos = str.Find(' ');
		if(pos == -1)
			word = str;
		else
			word = str.Left(pos);
	
		if(word.GetLength()>0)
			if(word[0] == '=')
			{
				// The word is encoded
				word = word.Mid(13);		// Remove the first chars
				word = word.Left(word.GetLength()-2);
				if(word[0] == 'Q')
				{
					//Qouted Printable
					word = word.Mid(2);		// Remove Q? char 
	
					for(int i=0; i < word.GetLength(); i++)
						if(word[i] == '_')
							word.SetAt(i, ' ');

					for(i=0; i < word.GetLength(); i++)
						if(word[i] == '=')
						{
							char ch = 0;
							char num1 = word[i+1]
								, num2 = word[i+2];
							if((num1 >= '0') &&
								(num1 <= '9'))
								ch = (num1 - '0') * 16;
							else
								ch = (num1 - 'A' + 10) * 16;
	
							if((num2 >= '0') &&
								(num2 <= '9'))
								ch += (num2 - '0');
							else
								ch += (num2 - 'A' + 10);
							i += 2;					// Handle the hex chars;
							res += ch;
						}
						else
						{
							res += word[i];
						}
				}
				else
				{
					// Base 64 encoding!
					res = "<Base64 encoding not implemented>";
				}
				res += ' ';
			}
			else
			{
				// The word is cleartext
				res += word + ' ';
			}

		if(pos == -1)
			str = "";
		else
		{
			str = str.Mid(pos);
			str.TrimLeft();
		}
	}
	return res;
}

void THeader::SetFlags(int inFlags)
{
	flags |= inFlags;
}

void THeader::ClearFlags(int inFlags)
{
	flags &= ~inFlags;
}

bool THeader::GetFlag(int inFlag)
{
	return (flags & inFlag) != 0;
}

TAddressList& THeader::GetTo()
{
	return *to;
}

TAddressList& THeader::GetCC()
{
	return *cc;
}

TAddress & THeader::GetFrom()
{
	return from;
}

CString THeader::GetSubject()
{
    return DecodeMIMEHeader(subject);
}

int THeader::GetFlags()
{
	return flags;
}

CString & THeader::GetDate()
{
	return datestr;
}

CTime THeader::GetCTime()
{
	return date;
}
