// H E R M E S  M A I L
// Copyright 1997-98 OaS 2004
// Further info at http://www.lysator.liu.se/~fcl/hermes/

// TAddressBook.cpp: implementation of the TAddressBook class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "hermes gui.h"
#include "TAddressBook.h"
#include "wabobject.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TAddressBook::TAddressBook()
{
	// Create the address list
	addresses = new TAddressList;
}

TAddressBook::~TAddressBook()
{
	if(m_pWabObject)
	{
		m_pWabObject->FreeWABCache(addresses);
		delete m_pWabObject;
	}
}

bool TAddressBook::AddAddress(HWND hWnd)
{
 HRESULT hr;
	if(m_pWabObject)
	{
		if((hr = m_pWabObject->ShowNewEntryDialog(hWnd)) == S_OK)
		{
			// Since we got a result indicating that the address book have been changed, 
			// we have to
			// invalidate the whole address book. That is, free all memory connected
			// to it and reload it all.

			m_pWabObject->FreeWABCache(addresses);
			delete m_pWabObject;

			addresses->RemoveAll();
		
			GetWABContents();
		}
		else
		{
			AfxMessageBox("No new address was created.");
		}
	}
	
	return hr == S_OK;
}

bool TAddressBook::RemoveAddress(TAddress * address)
{
	HRESULT hr;

	if(m_pWabObject)
	{
		hr = m_pWabObject->DeleteEntry(address);

		if(hr == S_OK)
		{
			m_pWabObject->FreeWABCache(addresses);
			delete m_pWabObject;

			addresses->RemoveAll();
		
			GetWABContents();
		}
	}
	return hr == S_OK; 
}

TAddress* TAddressBook::Lookup(const CString fullName) const
{
	char buf[250];
	TAddress* res;

	strncpy(buf, fullName, 249);
	buf[249] = '\0';

	if(m_pWabObject->LookupName(CString(buf), res) != S_OK)
	{
		return (TAddress*) NULL;
	}

	return res;
}

const TAddressList& TAddressBook::GetAddressList()
{
   return *addresses;
}

TAddressBook::TAddressBook(TAddressList * addressList)
{

}

void TAddressBook::GetWABContents()
{
   	m_pWabObject = new CWAB( &CString("") );

	if(m_pWabObject)
		m_pWabObject->LoadWABContents(addresses);
}

void TAddressBook::AddressProperties(TAddress * address, HWND hWnd)
{
	if(m_pWabObject && (address->data != NULL))
	{
		m_pWabObject->ShowSelectedItemDetails(address, hWnd);

		// Since we dont know if the address book have been changed or not
		// (and it doesnt seem to exist an easy solution to this), we have to
		// invalidate the whole address book. That is, free all memory connected
		// to it and reload it all.

		m_pWabObject->FreeWABCache(addresses);
		delete m_pWabObject;

		addresses->RemoveAll();
		
		GetWABContents();
	}
}

void TAddressBook::FindAddress(HWND hWnd)
{
	if(m_pWabObject)
		if(m_pWabObject->FindUser(hWnd) != S_OK)
		{
			AfxMessageBox("Unable to open find dialog!");
		}
}

// Check if a given string is a valid email address.
// A string is considered as a valid address if it
//   a) Contains exactly ONE '@'-symbol
//   b) Does NOT contain spaces or commas
//   c) Has text on both sides of the '@' symbol
//
// This means that !)(/")(/"#)(/@SE vill be evaluated to a valid
// email address, but what the heck....
bool TAddressBook::IsEmailAddress(const CString & inString)
{
	int atIndex;

	CString str = inString;	// Copy the instring
	str.TrimLeft();		// Get rid of redundant whitespaces
	str.TrimRight();

	atIndex = str.Find('@');
	if(atIndex == -1) return false;
	// Check left of @-sign
	if((str.Left(atIndex)).Find('@') != -1) return false;
	// Check right of @-sign
	if((str.Mid(atIndex+1)).Find('@') != -1) return false;

	// ok. Now we have a well-formed possible address of the form
	// leftstring@rightstring
	
	// Check for spaces
	if(str.Find(' ') != -1) return false;

	// Make sure the @-sign is INSIDE the string, and not the
	// left- or rightmost character...
	if((atIndex == 0) || (atIndex >= (str.GetLength()-1))) return false;

	// All tests passed.... We _probably_ have a valid email address...

	return true;
}
