// H E R M E S  M A I L
// Copyright 1997-98 OaS 2004
// Further info at http://www.lysator.liu.se/~fcl/hermes/

// $Header: /Hermes GUI/TFolder.cpp 23    97-12-11 0:10 Fcl $
// $Log: /Hermes GUI/TFolder.cpp $
//
//23    97-12-11 0:10 Fcl
//Fixed a bug when deleting a folder not yet saved + added removal from
//parent
//
//22    97-11-29 13:11 Fcl
//Added rename possibility
//
//21    97-11-26 13:04 Peblin
//fixed precompiled headers
//
//20    97-11-24 14:25 Peblin
//Removed A LOT of LoadLetters(). 
//
//19    97-11-24 9:28 Peblin
//Added LoadLetters() in Save()
//
//18    97-11-23 20:51 Peblin
//Gets and saves letters.
//
//17    97-11-23 13:27 Peblin
//Added CSocket support
//
//16    97-11-15 21:40 Fcl
//First tested version. Is able to save and load folders with no letters
//inside
//
//15    11/15/97 7:32p Andreas
//
//14    11/15/97 7:30p Andreas
//
//13    97-11-15 13:19 Bjorn
//Added GetFLags() for TIO
//
//12    97-11-15 12:35 Bjorn
//Added SetID (for TIO class)
//
//11    97-11-14 11:20 Fcl
//Changed TMMap::GetPointer to return a constant pointer and moved
//postOffice decl.
//
//10    11/13/97 17:00 Magnus
//All fuctionality in TFolder implemented.
//
//9     97-11-11 20:25 Peblin
//Added and implemented the GetID() function
//
//8     97-11-06 21:59 Fcl
//Added Header and Log statements at the top of each file

// TFolder.cpp: implementation of the TFolder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TFolder.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TFolder& NO_FOLDER = *((TFolder*) NULL);

TFolder::TFolder(CString &inName, int id)
{
	name = inName;	//Sets the name of the folder
	folderID = id;
	letters = new TLetterList;
	folders = new TFolderList;
	parent = NULL; // Folder has no parent
	flags = 0; // No flags set
//	Save();
}

TFolder::~TFolder()
{
	TLetter* l;
	TFolder* f;
	POSITION pos;

	// Remove letters in folder
	pos = letters->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		l = letters->GetNext(pos);
		delete l;
	}
	// Clear the LetterList
	letters->RemoveAll();

	// Remove subfolders
	pos = folders->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		f = folders->GetNext(pos);
		delete f;
	}
	// Clear the LetterList
	folders->RemoveAll();

	// NOT in design ??

	if(parent != NULL)
		parent->RemoveSubFolder(*this);

	delete letters;
	delete folders;
}

void TFolder::Save()
{
	POSITION pos;
	TFolder* f;
	TLetter* l;

//	LoadLetters();	// Letters need to be loaded in order to be "saveable"	
	// Save letters if modified
	pos = letters->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		l = letters->GetNext(pos);
		if (l->GetHeader().GetFlag(THeader::changed)) 
			l->Save();
	}
	
	// Save subfolders if modified
	pos = folders->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		f = folders->GetNext(pos);
		if (f->GetFlag(changed))  //Fix GetFlags()!!!!!!!!!!!
			f->Save();
	}

	// Save myself if modified
	if (GetFlag(changed)) 
	{
		postOffice.GetIO().SaveFolder(*this);
		ClearFlags(changed);
	}
}

void TFolder::Rename(CString& inName)
{
//	LoadLetters();
	name = *new CString(inName);
	SetFlags(changed); // Set changed flag
}

void TFolder::ChangeParent(TFolder& inParent) //Are not compliant with the design!!!!!
{
//	LoadLetters();
	parent = &inParent; // Set new parent
	SetFlags(changed); // Set changed flag
}

void TFolder::AddLetter(TLetter& letter)
{
//	LoadLetters();
	letters->AddTail(&letter);
	letter.SetFolder(*this);
	SetFlags(changed); // Set changed flag
	
}

void TFolder::RemoveLetter(TLetter& letter)
{
//	LoadLetters();
	// Remove letter from folder. If letter not in folder throw Exception
	POSITION pos =  letters->Find(&letter); 
	if (pos == NULL)
		EXCEPT("Letter not in folder!");
	else
		letters->RemoveAt(pos);
	
	// Letter has no folder
	letter.SetFolder(NO_FOLDER);
}

void TFolder::AddSubFolder(TFolder& folder)
{
	// Add subfolder to folder
	folders->AddTail(&folder);
	folder.ChangeParent(*this);
	SetFlags(changed); // Set changed flag
}

void TFolder::LoadLetters()
{
	// If the letters aren't loaded from disk then do that!
	if (!(GetFlag(lettersloaded)))
	{
		postOffice.GetIO().LoadHeaders(*this);
	    SetFlags(lettersloaded);
	}
}

TLetterList& TFolder::GetLetters()
{
//	LoadLetters();
	return *letters;
}

void TFolder::Delete()
{

	// Remove letters and folder(subfolders too!) on disk
	POSITION pos;
	TFolder* f;
	TLetter* l;

//	LoadLetters();
	
	pos = letters->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		l = letters->GetNext(pos);
		l->Delete();
	}
	
	
	pos = folders->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		f = folders->GetNext(pos);
		f->Delete();
	}
	// Remove myself if on disk
	if(folderID != NO_ID)
		postOffice.GetIO().DeleteFolder(*this);
}



int TFolder::GetID()
{
	return folderID;
}

TFolder& TFolder::GetParent()
{
	return *parent;
}

TFolderList& TFolder::GetSubFolders()
{
	return *folders;
}

void TFolder::RemoveSubFolder(TFolder & folder)
{
	
	
//	LoadLetters();
	
	// Remove subfolder from folder. If subfolder not in folder throw Exception
	POSITION pos =  folders->Find(&folder); 
	if (pos == NULL)
		EXCEPT("Folder not in folder!");
	else
	{
		folders->RemoveAt(pos);
		folder.ChangeParent(NO_FOLDER);
	}
}


// Needed for CList to work
TFolder::TFolder()
{

}

void TFolder::SetFlags(int inFlags)
{
	flags |= inFlags;
}

void TFolder::ClearFlags(int inFlags)
{
	flags &= ~inFlags;
}

bool TFolder::GetFlag(int inFlag)
{
	return (inFlag & flags) != 0;
}

CString& TFolder::GetName()
{
	return name;
}

void TFolder::SetID(int ID)
{
	folderID = ID;
}

int TFolder::GetFlags()
{
	return flags;
}