// H E R M E S  M A I L
// Copyright 1997-98 OaS 2004
// Further info at http://www.lysator.liu.se/~fcl/hermes/

// $Header: /Hermes GUI/TLetter.cpp 22    98-01-25 12:10 Bjorn $
// $Log: /Hermes GUI/TLetter.cpp $
//
//22    98-01-25 12:10 Bjorn
//Added sourcecomments for examination
//
//21    97-11-29 16:01 Fcl
//
//20    97-11-29 13:11 Fcl
//Added rename possibility
//
//19    97-11-28 23:38 Fcl
//Added cut&paste support in header window
//Added copy constructors when needed,
//
//18    97-11-28 23:07 Fcl
//Removed a comment
//
//17    97-11-28 10:27 Fcl 
//Fixed a bug in the constructor called from GUI which didnt set
//the changed flag and thus the new letter weren't saved
//
//16    97-11-26 13:04 Peblin
//fixed precompiled headers
//
//15    97-11-23 20:51 Peblin
//Gets and saves letters.
//
//14    97-11-23 13:27 Peblin
//Added CSocket support
//
//13    97-11-14 19:31 Fcl
//Check in to be able to move the repository to mandel.
//
//12    11/13/97 17:02 Magnus
//Changed * to & in constuctor.
//
//11    97-11-12 17:34 Fcl
//IMPORTANT!!!
//We added a new input argument. TRawData.....
//
//10    11/09/97 14:11 Magnus
//Fixed rawData pointer in constructor.
//
//9     97-11-08 18:42 Fcl
//Added comment about the creation of RawData
//
//8     11/08/97 18:01 Magnus
//Fixed TLetter constructor.
//
//7     11/08/97 17:45 Magnus
//All functionality implemented. Compiles.
//
//6     97-11-06 21:59 Fcl
//Added Header and Log statements at the top of each file

// TLetter.cpp: implementation of the TLetter class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "TLetter.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern TPostOffice postOffice;

TLetter::~TLetter()
{

}


/* Constructor
 * Call this function to create a new letter from file. -
 */
TLetter::TLetter(int inID, THeader & inHeader, TRawData &inRawData)
{
	folder = NULL;	// A new letter, use SetFolder to set parent-folder
	header = &inHeader;
	ID = inID;
	rawData = &inRawData;
}

/* Constructor
 * Call this function to create a new letter from 
 * encoded data. (e.g. incoming mail)
 */
TLetter::TLetter(TRawData & inRawData)
{
	folder = NULL;  

	//Decode the incoming rawdata to a header using the global postOffice-object
	header = &postOffice.GetCodeManager().DecodeHeader(inRawData);

	ID = NO_ID; // The new letter is'nt linked to a file yet
	            // Use save to secure the data

	//The letterdata now differs from filedata (since there is no file yet)
	header->SetFlags(THeader::changed);
	rawData = &inRawData;
}

/* Constructor
 * Call this function to create a letter from 
 * pure text (e.g. when you write a new one)
 * Usually called from GUI - the letter should be saved later.
 */
TLetter::TLetter(THeader & inHeader, CString & s)
{
	folder = NULL;   
	header = &inHeader;

	//No file is linked to it yet
	header->SetFlags(THeader::changed);
	ID = NO_ID; 

	//Encode the incoming textstring to the correct mailformat
	//the rawdata is now just a mempointer
	rawData = new TRawData((postOffice.GetCodeManager()).EncodeLetter(s, inHeader));
}

// Changed SetFolderID to SetFolder
void TLetter::SetFolder(TFolder& newFolder) 
{
	folder = &newFolder;
}

//Changed GetFolderID to GetFolder
TFolder& TLetter::GetFolder() 
{
	return *folder;
}

CString & TLetter::GetReadableText()
{
	return (postOffice.GetCodeManager()).DecodeLetter(*rawData);
}

THeader & TLetter::GetHeader()
{
	return *header;
}

TRawData & TLetter::GetRawData()
{
	return *rawData;
}

void TLetter::Save()
{
	// Only save the letter if modified
	if(GetHeader().GetFlag(THeader::changed))
	{
		postOffice.GetIO().SaveLetter(*this);
	}
}

// Returns the complete dataheader in textformat
CString & TLetter::GetBlaBlaBla()
{
	return (postOffice.GetCodeManager()).GetBlaBlaBla(*rawData);
}

void TLetter::Delete()
{
	//Remove the letter from file as well as from parent-folder -
	postOffice.GetIO().DeleteLetter(*this);
	GetFolder().RemoveLetter(*this);
}

void TLetter::SaveAttachment(TAttachment attachment, CString filename)
{
	(postOffice.GetIO()).SaveAttachment(attachment, filename);
}

int TLetter::GetID()
{
	return ID;
}
	
// SetID - used by TIO when saving new letters
void TLetter::SetID(int inID)
{
		ID = inID;
}


//Copy constructor
TLetter::TLetter(TLetter & oldLetter)
{
	folder = NULL; // When a letter is copied the folder should be none.
	header = new THeader(*oldLetter.header);
	ID = NO_ID;
	rawData = new TRawData(*oldLetter.rawData);
	header->SetFlags(THeader::changed);

}

