// H E R M E S  M A I L
// Copyright 1997-98 OaS 2004
// Further info at http://www.lysator.liu.se/~fcl/hermes/

// TMIMEDecoder.cpp: implementation of the TMIMEDecoder class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "TMIMEDecoder.h"
#include "mimepp/mimepp.h"
#include "mimep100/multipar.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TMIMEDecoder::TMIMEDecoder()
{

}

TMIMEDecoder::~TMIMEDecoder()
{

}

CString& TMIMEDecoder::Decode(TRawData& rawData)
{  
    CString decodedString;
	
	//Initialize the Mime++
	DwInitialize(); 

	// Create a message object
	DwMessage* msg = DwMessage::NewMessage(DwString(rawData.GetMem(), rawData.GetSize()), 0);
	
	// Execute the parse method, which will create the broken-down
	// representation (the tree representation, if you recall)
	msg->Parse();
	
	// Check if multipart
	if (msg->Headers().ContentType().Type() == DwMime::kTypeMultipart) 
	{
		decodedString += DecodeMultipart(msg->Body());
    }
	else
	{
		// Not Multipart
		// Check if it has an encodig type
		if (msg->Headers().HasContentTransferEncoding()) 
		{
			int cte = msg->Headers().ContentTransferEncoding().AsEnum();
			// If QuotedPrintable -> Decode it.
			if (cte == DwMime::kCteQuotedPrintable)
			{
				DwString body;
				DwDecodeQuotedPrintable(msg->Body().AsString(), body);
				decodedString += CString(body.c_str());
			}
			else
				// If Base64 -> Decode it.
				if (cte == DwMime::kCteBase64)
				{
					DwString body;
					DwDecodeBase64(msg->Body().AsString(), body);
					decodedString += CString(body.c_str());
				}
				else
				{
					// Other type than QuotedPrintable or Base64 -> Return as it is.
					const char* tmpString = msg->Body().AsString().c_str();
					decodedString += CString(tmpString);
				}
		}
		else
		{
			// No encoding type given -> Return as it is.
			const char* tmpString = msg->Body().AsString().c_str();
			decodedString += CString(tmpString);
		}
	}
//	orgString = rawText.Body().AsString().c_str();

	return *(new CString(decodedString));  
}


CString& TMIMEDecoder::DecodeMultipart(DwBody& multibody)
{
	CString decodedString;
	DwString body;
	
	// For all parts in multipart
    DwBodyPart* part = multibody.FirstBodyPart();
    while (part) 
	{
		if (part->Headers().ContentType().Type() == DwMime::kTypeMultipart)
			decodedString += DecodeMultipart(part->Body());
		else
		{
			// Not Multipart
			// Check if it has an encoding type
			// and if it is text
			if ((part->Headers().HasContentTransferEncoding()) &&
				(part->Headers().ContentType().Type() ==  DwMime::kTypeText) &&
				(!(part->Headers().ContentType().Subtype() ==  DwMime::kSubtypeHtml)))
			{
				int cte = part->Headers().ContentTransferEncoding().AsEnum();
				// If QuotedPrintable -> Decode it.
				if (cte == DwMime::kCteQuotedPrintable)
				{
					DwString body;
					DwDecodeQuotedPrintable(part->Body().AsString(), body);
					decodedString += CString(body.c_str());
				}
				else
					// If Base64 -> Decode it.
					if (cte == DwMime::kCteBase64)
					{
						DwString body;
						DwDecodeBase64(part->Body().AsString(), body);
						decodedString += CString(body.c_str());
					}
					else
					{
						// Other type than QuotedPrintable or Base64 -> Return as it is.
						const char* tmpString = part->Body().AsString().c_str();
						decodedString += CString(tmpString);
					}
			}
			else
			// The part either has no CTE or it is not text  
			{
				// No encoding type given -> Return as it is.
				if (!(part->Headers().HasContentTransferEncoding()))
				{
					const char* tmpString = part->Body().AsString().c_str();
					decodedString += CString(tmpString);
				}
				// If it's not text do nothing
			}
		}
		// Get next part to go on with
		part = part->Next();
    }
   
	return *(new CString(decodedString));
}		
