// H E R M E S  M A I L
// Copyright 1997-98 OaS 2004
// Further info at http://www.lysator.liu.se/~fcl/hermes/

// $Header: /Hermes GUI/TPostOffice.cpp 32    98-01-25 17:25 Peblin $
// $Log: /Hermes GUI/TPostOffice.cpp $
//
//32    98-01-25 17:25 Peblin
//Commented the code
//
//31    97-12-10 20:37 Fcl
//
//30    97-12-09 21:15 Fcl
//Fixed misc. bugs in the TIO
//
//29    12/06/97 6:16p Andreas
//Added functionallity for handling preference.
//
//28    97-12-06 0:22 Fcl
//Fixed the startup - can now handle empty mail folder or corrupt one.
//
//27    97-12-02 23:43 Peblin
//Changed creation of TAddressBook.
//
//26    97-11-29 17:43 Peblin
//party
//
//25    97-11-29 10:55 Peblin
//added initialization code for TAddressBook.
//
//24    97-11-29 0:18 Fcl
//Fixed a () error
//
//23    97-11-28 23:51 Fcl
//Fixed a bug in PlaceFolder: forgotten paranthesis!
//
//22    97-11-28 0:05 Fcl
//Save the rootfolder with deleteing
//
//21    97-11-27 21:14 Fcl
//
//20    97-11-26 14:56 Peblin
//
//19    97-11-26 13:04 Peblin
//fixed precompiled headers
//
//18    97-11-23 20:51 Peblin
//Gets and saves letters.
//
//17    97-11-23 13:27 Peblin
//Added CSocket support
//
//16    97-11-22 17:16 Peblin
//fixed missning semicolon
//
//15    97-11-22 17:14 Peblin
//Added deletion of the letters variable in ReceiveLetters
//
//14    97-11-15 21:40 Fcl
//First tested version. Is able to save and load folders with no letters
//inside
//
//13    97-11-12 11:50 Peblin
//Added a LOT of exception handling. The more, the merrier :)
//Well probably have to remove some of it later....
//
//12    97-11-12 10:21 Peblin
//Implemented ListOutgoingMail(). Compiling version.
//
//11    97-11-11 23:03 Peblin
//Implemented ReceiveLetters(). Read the comments for the function!
//
//10    97-11-11 21:07 Peblin
//Implemented AddLetter and AddFolder. AddFolder prevents the user from
//moving a folder into one of its own subfolders. (Please verify the
//algorithm)
//
//9     97-11-11 20:37 Peblin
//Compiling version.
//
//8     97-11-11 20:30 Peblin
//See diff for changes
//
//7     97-11-11 18:52 Peblin
//Implemented a bunch of functions
//
//6     11/08/97 14:55 Magnus
//Added GetIO function. Needed for TLetter.
//
//5     97-11-06 21:59 Fcl
//Added Header and Log statements at the top of each file

// TPostOffice.cpp: implementation of the TPostOffice class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "TPostOffice.h"
#include "TGenericException.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TPostOffice::TPostOffice()
{
	// NO INITIATION HERE
	// Use Initialize() instead.
}

TPostOffice::~TPostOffice()
{
	rootFolder->Save();
	delete preference;
	delete addressBook;
	delete mailBox;
	delete ruleManager;
	delete codeManager;
	delete IO;
	delete rootFolder;
}

void TPostOffice::Initialize()
{
	// Create all Hermes objects 
	try 
	{
		IO = new TIO(".\\");		// The mail database is in the current directory = startup dir.
		codeManager = new TCodeManager();
		ruleManager = new TRuleManager();

	}
	catch(TGenericException e) 
	{
		e.ReportError();
		EXCEPT("Fatal error. Unable to create vital objects.");
	}

	// Load the root folder
	TRY_CATCH(LoadRootFolder());

	// Assign pointers for convenience. Assumes that all the folders
	// are created.
	outBox = &(FindFolder(outBoxID));
	inBox = &(FindFolder(inBoxID));
	sentMail = &(FindFolder(sentMailID));
	trash = &(FindFolder(trashID));
	
	// Check for the existence of neccessary folders
	if(outBox == NULL || inBox == NULL || sentMail == NULL || trash == NULL) 
	{
		EXCEPT("Fatal error. Missing vital system folders!");
	}

	// Create the address book and connect it to WAB.
	addressBook = new TAddressBook;
	addressBook->GetWABContents();

	// Get Preference from disk
	preference = IO->LoadPreference();

	// Create a new thread and start checking mail
	TRY_CATCH(mailBox = new TMailBox());
}

void TPostOffice::LoadRootFolder()
{
	// Load the root folder and assign the returned pointer to the folder
	// to rootFolder.
	try
	{
		rootFolder = &(IO->LoadFolder(rootID));
	}
	catch(...)
	{
		//Oops somethong went wrong - we have to recreate the file system
		AfxMessageBox("The mail database didn't load. Recreating from scratch.");
		rootFolder = &(IO->RecoverDatabase());
		rootFolder->Save();
	}
}

void TPostOffice::DeleteLetter(TLetter & letter)
{
	TRY_CATCH(letter.Delete()); // Safely try to delete letter.
	delete &letter; // According to the specs. NOTE: This is not explicitly
					// stated in the design, but TLetter::Delete() should
					// have the same structure as TFolder::Delete() for
					// consistency.	/ Per
}

void TPostOffice::DeleteFolder(TFolder & folder)
{
	// According to the design, this function calls folder.Delete(). This
	// implies that it should NOT be used to clean up after the program.
	// The following exceptions will therefore prevent the user from
	// deleting the necessary folders.		/ Per
	if( folder.GetID() == rootID ||
		folder.GetID() == inBoxID ||
		folder.GetID() == outBoxID ||
		folder.GetID() == sentMailID ||
		folder.GetID() == trashID)
	{
		EXCEPT("Deletion of system folders not allowed");
		return;  // Terminate execution of the function
	}

	try 
	{
		folder.Delete(); // Remove from disk
	}	
	catch(TGenericException e) 
	{
		e.ReportError();
		EXCEPT("Unable to delete folder");
		return;
	}
	delete &folder;		// Remove from memory
}

TFolder& TPostOffice::FindFolder(int id)
{
	// Locate the folder using TIO
   return IO->FindFolder(id);
}

void TPostOffice::PlaceLetter(TLetter & letter, TFolder & folder)
{
	if(&(letter.GetFolder()) != NULL)
		letter.GetFolder().RemoveLetter(letter);
	
	folder.AddLetter(letter);

	// Send the letter (and possibly, waiting mail in the outBox)
	if(folder.GetID() == outBoxID) 
	{
		mailBox->PrepareToSend();
	}
}

void TPostOffice::PlaceFolder(TFolder & folder, TFolder & dest)
{
 TFolder* f;

	// The following algorithm detects and prevents the disaster
	// that would be caused if a folder was moved into its own
	// subtree.   /Per

	// Search the up-tree build by the parent-pointers. The algorithm
	// begins at the dest pointer, traverses up the tree until 
	// (1) the current node matches the folder pointer OR 
	// (2) the current node is assigned the NULL pointer.
	// If case 1 happens, an exception is generated.
	// If case 2 happens, all is OK.
	f = &dest;
	while(!(f == NULL)) 
	{
		if(f == &folder) 
		{
			EXCEPT("A folder can not be moved into its own subfolder!");
		}
		f = &(f->GetParent());
	}

	// If the folder already has a parent, remove it from it.
	if((&folder.GetParent()) != NULL) {
		folder.GetParent().RemoveSubFolder(folder);
	}

	// Add the folder to its new parent.
	dest.AddSubFolder(folder);
}

void TPostOffice::ReceiveLetters(TLetterList& letters)
{
 POSITION pos;
 TLetter* l;
 THeader* h;

	pos = letters.GetHeadPosition();
	while(!(pos == NULL)) {
		l = letters.GetNext(pos); // retrive "pos"-element and increment pos
		h = &(l->GetHeader());
		
		// Place letter according to the status of it.
		if( h->GetFlag( THeader::sent )) 
		{  
			// The letter was successfully sent
			PlaceLetter(*l, *sentMail);
		} 
		else if( h->GetFlag( THeader::external )) 
		{  
			// The letter comes from the outside world
			PlaceLetter(*l, *inBox);
		} 
		else 
		{  
			// The letter could not be sent. Replace in outBox.
			// NOTE: This is *stupid* behavior. PlaceLetter() will immediately cause a 
			// PrepareToSend() in the mailBox, which will result in the mailBox trying to
			// resend the message within milliseconds after its told the postOffice that
			// sending was impossible. Duh.	The fix doesnt seem to be worth the
			// trouble though... / Per
			PlaceLetter(*l, *outBox);
		}
	}
	
	// Delete the list of letters (all the letters have been taken care of).
	delete &letters;
}

TLetterList& TPostOffice::ListOutgoingMail()
{
 POSITION pos;
 TLetter* letter;
 TLetterList& llist=outBox->GetLetters(); // Assign all letters in outbox to llist
 TLetterList* savelist;

	// Allocate a new list. THIS SHOULD BE DESTROYED IN TMAILBOX!
	savelist = new TLetterList();
	
	// Make a copy of llist. No copy constructor exists.
	pos = llist.GetHeadPosition();
	while(!(pos == NULL)) 
	{
		letter = llist.GetNext(pos);
		savelist->AddTail(letter);
	}
	
	pos = savelist->GetHeadPosition();
	while(!(pos == NULL)) 
	{
		letter = savelist->GetNext(pos);
		// Remove the letter from the folder.
		// This should change the contents of llist. (Hence the savelist)
		try 
		{
			outBox->RemoveLetter(*letter);
		}
		catch(TGenericException e) 
		{
			e.ReportError();
			EXCEPT("Strange exception! Letter in outBox did not belong to outbox!!??");
		}
	}

	// Give the list of letters to the caller.
	return *savelist;
}

//////////////////////////////////////////////////////////////////////////////////////////
// Data access functions
//////////////////////////////////////////////////////////////////////////////////////////

TFolder& TPostOffice::GetRootFolder()
{
   return *rootFolder;
}

TRuleManager& TPostOffice::GetRuleManager()
{
   return *ruleManager;
}

TCodeManager& TPostOffice::GetCodeManager()
{
   return *codeManager;
}

TIO& TPostOffice::GetIO()
{
	return *IO;
}

TMailBox & TPostOffice::GetMailBox()
{
	return *mailBox;
}

TAddressBook* TPostOffice::GetAddressBook()
{
	return addressBook;
}

TPreference& TPostOffice::GetPreference()
{
	return *preference;
}
